@@ -17,6 +17,8 @@ our @EXPORT = qw(get_proxy
 	set_proxy_config dump_proxy_config
 );
 
+(our $VERSION) = q($Revision: 271299 $) =~ /(\d+)/;
+
 #- proxy config file.
 our $PROXY_CFG = '/etc/urpmi/proxy.cfg';
 my $proxy_config;
@@ -25,18 +27,6 @@ my $proxy_config;
 our $CONNECT_TIMEOUT = 60; #-  (in seconds)
 
 
-=head1 NAME
-
-urpm::download - download routines for the urpm* tools
-
-=head1 SYNOPSIS
-
-=head1 DESCRIPTION
-
-=over
-
-=cut
-
 
 sub ftp_http_downloaders() { qw(curl wget prozilla aria2) }
 
@@ -93,6 +83,13 @@ sub preferred_downloader {
 	    $urpm->{log}(N("%s is not available, falling back on %s", $requested_downloader, $preferred));
 	}
     }
+    # in some cases, we may want not to use metalink (aria2) since it can 
+    # cause issues, see #53434 for example
+    if ($metalink_disabled && member($preferred, @metalink_downloaders)) {
+	$urpm->{log}("not using $preferred since metalink has been disabled");
+	my @no_metalink = urpm::util::difference2(\@available, \@metalink_downloaders);
+	$preferred = $no_metalink[0];
+    }
 
     if ($$use_metalink && !member($preferred, @metalink_downloaders)) {
 	$warned{not_using_metalink}++ or 
@@ -164,21 +161,16 @@ sub remove_proxy_media {
 }
 
 sub get_proxy_ {
-    my ($urpm) = @_;
+    my ($urpm, $_medium) = @_;
 
     -e $PROXY_CFG && !-r $PROXY_CFG and $urpm->{error}(N("can not read proxy settings (not enough rights to read %s)", $PROXY_CFG));
 
     get_proxy($urpm);
 }
 
-=item get_proxy($media)
-
-Reads and loads the proxy.cfg file ;
-Returns the global proxy settings (without arguments) or the
-proxy settings for the specified media (with a media name as argument)
-
-=cut
-
+#- reads and loads the proxy.cfg file ;
+#- returns the global proxy settings (without arguments) or the
+#- proxy settings for the specified media (with a media name as argument)
 sub get_proxy (;$) {
     my ($o_media) = @_; $o_media ||= '';
     load_proxy_config();
@@ -222,12 +214,7 @@ sub copy_cmd_line_proxy {
     }
 }
 
-=item set_cmdline_proxy(%h)
-
-Overrides the config file proxy settings with values passed via command-line
-
-=cut
-
+#- overrides the config file proxy settings with values passed via command-line
 sub set_cmdline_proxy {
     my (%h) = @_;
     load_proxy_config();
@@ -240,12 +227,7 @@ sub set_cmdline_proxy {
     $proxy_config->{cmd_line}{$_} = $h{$_} foreach keys %h;
 }
 
-=item set_proxy_config($key, $value, $o_media)
-
-Changes permanently the proxy settings
-
-=cut
-
+#- changes permanently the proxy settings
 sub set_proxy_config {
     my ($key, $value, $o_media) = @_;
     $proxy_config->{$o_media || ''}{$key} = $value;
@@ -508,14 +490,14 @@ sub sync_curl {
 	    "--stderr", "-", # redirect everything to stdout
 	    @all_files);
 	$options->{debug} and $options->{debug}($cmd);
-	$result = _curl_action($cmd, $options, @l);
+	$result = _curl_action($cmd,$options,@l);
     }
     chdir $cwd;
     $result;
 }
 
 sub _curl_action {
-    my ($cmd, $options, @l) = @_;
+    my ($cmd, $options, @l, $o_is_upload) = @_;
     
 	my ($buf, $file); $buf = '';
 	my $curl_pid = open(my $curl, "$cmd |");
@@ -534,6 +516,8 @@ sub _curl_action {
 			if (propagate_sync_callback($options, 'progress', $file, $percent, $total, $eta, $speed) eq 'canceled') {
 			    kill 15, $curl_pid;
 			    close $curl;
+			    
+			    die N("curl failed: upload canceled\n") if $o_is_upload;
 			    die N("curl failed: download canceled\n");
 			}
 			#- this checks that download has actually started
@@ -692,24 +676,19 @@ sub sync_aria2 {
 
     my $aria2c_command = join(" ", map { "'$_'" }
 	"/usr/bin/aria2c", $options->{debug} ? ('--log', "$options->{dir}/.aria2.log") : @{[]},
-	'--auto-file-renaming=false',
+	"--auto-file-renaming=false",
 	'--ftp-pasv',
-	'--summary-interval=0',
-	'--follow-metalink=mem',
+	"--follow-metalink=mem",
       $medium->{mirrorlist} ? (
-	'--metalink-enable-unique-protocol=true', # do not try to connect to the same server using the same protocol
-	 '--metalink-preferred-protocol=http', # try http as first protocol as they're stateless and
-	                                       # will put less strain on ie. the ftp servers which connections
-	                                       # are statefull for, causing unhappy mirror admins complaining
-	                                       # about increase of connections, increasing resource usage.
-	'--max-tries=5', # nb: not using $options->{retry}
+	'--metalink-enable-unique-protocol=false', # so that it can try both ftp and http access on the same server. aria2 will only do this on first calls
+	'--max-tries=1', # nb: not using $options->{retry}
 	'--lowest-speed-limit=20K', "--timeout", 3,
-        '--split=3', # maximum number of servers to use for one download
+        '--metalink-servers=3', # maximum number of servers to use for one download
         '--uri-selector=adaptive', "--server-stat-if=$stat_file", "--server-stat-of=$stat_file",
-        $options->{is_versioned} ? @{[]} : '--max-file-not-found=9', # number of not found errors on different servers before aborting file download
+        $options->{is_versioned} ? @{[]} : '--max-file-not-found=3', # number of not found errors on different servers before aborting file download
         '--connect-timeout=6', # $CONNECT_TIMEOUT,
       ) : @{[]},
-	'-Z', '-j1',
+	"-Z", "-j1",
 	($options->{'limit-rate'} ? "--max-download-limit=" . $options->{'limit-rate'} : @{[]}),
 	($options->{resume} ? "--continue" : "--allow-overwrite=true"),
 	($options->{proxy} ? set_proxy({ type => "aria2", proxy => $options->{proxy} }) : @{[]}),
@@ -823,25 +802,20 @@ eval {
 };
 
 sub progress_text {
-    my ($mode, $percent, $total, $eta, $speed) = @_;
+    my ($mode, $_file, $percent, $total, $eta, $speed) = @_;
     $mode eq 'progress' ?
       (defined $total && defined $eta ?
 	 N("        %s%% of %s completed, ETA = %s, speed = %s", $percent, $total, $eta, $speed) :
 	 N("        %s%% completed, speed = %s", $percent, $speed)) : '';
 }
 
-=item sync_logger($mode, $file, $percent, $_total, $_eta, $_speed)
-
-Default logger (callback) suitable for sync operation on STDERR only.
-
-=cut
-
+#- default logger suitable for sync operation on STDERR only.
 sub sync_logger {
-    my ($mode, $file, $percent, $total, $eta, $speed) = @_;
+    my ($mode, $file, $percent, $_total, $_eta, $_speed) = @_;
     if ($mode eq 'start') {
 	print STDERR "    $file\n";
     } elsif ($mode eq 'progress') {
-	my $text = progress_text($mode, $percent, $total, $eta, $speed);
+	my $text = &progress_text;
 	if (length($text) > $wchar) { $text = substr($text, 0, $wchar) }
 	if (bytes::length($text) < $wchar) {
 	    # clearing more than needed in case the terminal is not handling utf8 and we have a utf8 string
@@ -858,12 +832,6 @@ sub sync_logger {
     }
 }
 
-=item requested_ftp_http_downloader($urpm, $medium)
-
-Return the downloader program to use (whether it pas provided on the
-command line or in the config file).
-
-=cut
 
 sub requested_ftp_http_downloader {
     my ($urpm, $medium) = @_;
@@ -894,7 +862,7 @@ sub _all_options {
 
     my %all_options = ( 
 	dir => "$urpm->{cachedir}/partial",
-	proxy => get_proxy_($urpm),
+	proxy => get_proxy_($urpm, $medium),
 	metalink => $medium->{mirrorlist},
 	$medium->{"disable-certificate-check"} ? "no-certificate-check" : @{[]},
 	$urpm->{debug} ? (debug => $urpm->{debug}) : @{[]},
@@ -939,12 +907,6 @@ sub sync_rel_one {
     $files->[0];
 }
 
-=item sync_url($urpm, $url, %options)
-
-Retrieve a file from the network and return the local cached file path.
-
-=cut
-
 sub sync_url {
     my ($urpm, $url, %options) = @_;
     sync_rel_one($urpm, { url => dirname($url), disable_metalink => $options{disable_metalink} }, basename($url), %options);
@@ -959,11 +921,19 @@ sub sync_rel_to {
     $result_file;
 }
 
-=item get_content($urpm, $url, %o_options)
+#- deprecated, use sync_url() or sync_rel() instead
+#-
+#- $medium can be undef
+#- known options: quiet, resume, callback, ask_retry
+sub sync {
+    my ($urpm, $medium, $files, %options) = @_;
 
-Retrieve a file and return its content.
-
-=cut
+    if ($medium) {
+	$urpm->{error}("deprecated urpm::download::sync() called with a medium, this is not handled anymore, not using the medium and only taking the protocol into account");
+    }
+    sync_url($urpm, $_, %options) or return foreach @$files;
+    1;
+}
 
 sub get_content {
     my ($urpm, $url, %o_options) = @_;
@@ -1089,7 +1059,13 @@ sub _create_metalink_ {
 
 __END__
 
-=back
+=head1 NAME
+
+urpm::download - download routines for the urpm* tools
+
+=head1 SYNOPSIS
+
+=head1 DESCRIPTION
 
 =head1 COPYRIGHT
 
