@@ -6,7 +6,6 @@ use strict;
 no warnings;
 use Exporter;
 use URPM;
-use urpm::util;
 
 my $encoding;
 BEGIN {
@@ -14,8 +13,10 @@ BEGIN {
     eval "use open ':locale'" if $encoding && $encoding ne 'ANSI_X3.4-1968';
 }
 
+(our $VERSION) = q($Revision: 271299 $) =~ /(\d+)/;
+
 our @ISA = 'Exporter';
-our @EXPORT = qw(N N_ P translate bug_log message_input toMb formatXiB sys_log);
+our @EXPORT = qw(N N_ P translate bug_log message_input message_input_ toMb formatXiB sys_log);
 
 #- I18N.
 use Locale::gettext;
@@ -93,15 +94,23 @@ sub sys_log { defined &syslog and eval { syslog("info", @_) } }
 
 #- writes only to logfile, not to screen
 sub bug_log {
-    append_to_file($::logfile, @_) if $::logfile;
+    if ($::logfile) {
+	open my $fh, ">>$::logfile"
+	    or die "Can't output to log file [$::logfile]: $!\n";
+	print $fh @_;
+	close $fh;
+    }
 }
 
 sub ask_yes_or_no {
     my ($msg) = @_;
-    message_input($msg . N(" (y/N) "), boolean => 1) =~ /[$yesexpr]/;
+    message_input_($msg . N(" (y/N) "), boolean => 1) =~ /[$yesexpr]/;
 }
 
-sub message_input {
+#- deprecated, use message_input_() instead
+sub message_input { &_message_input }
+
+sub message_input_ {
     my ($msg, %o_opts) = @_;
     _message_input($msg, undef, %o_opts);
 }
